@@ -3,7 +3,7 @@
 ;; Copyright (C) 2011-2016  Guillaume Papin
 
 ;; Author: Guillaume Papin <guillaume.papin@epitech.eu>
-;; Version: 1.2.0
+;; Version: 1.6.1
 ;; URL: https://github.com/Sarcasm/irony-mode
 ;; Compatibility: GNU Emacs 24.x
 ;; Keywords: c, convenience, tools
@@ -97,6 +97,11 @@ automatically buffer-local wherever it is set."
   :type 'string
   :group 'irony)
 
+(defcustom irony-extra-cmake-args nil
+  "Extra arguments to CMake when compiling the server."
+  :type '(repeat string)
+  :group 'irony)
+
 (defcustom irony-user-dir (locate-user-emacs-file "irony/")
   "Directory containing the Irony generated files.
 
@@ -173,7 +178,9 @@ the buffer size to use for the irony-server process pipe on Windows.
 Larger values can improve performances on large buffers.
 
 If non-nil, `w32-pipe-buffer-size' will be let-bound to this value
-during the creation of the irony-server process.")
+during the creation of the irony-server process."
+  :type 'integer
+  :group 'irony)
 
 
 ;;
@@ -399,9 +406,9 @@ If no such file exists on the filesystem the special file '-' is
 ;;;###autoload
 (define-minor-mode irony-mode
   "Minor mode for C, C++ and Objective-C, powered by libclang."
-  nil
-  irony-lighter
-  irony-mode-map
+  :init-value nil
+  :lighter irony-lighter
+  :keymap irony-mode-map
   :group 'irony
   (if irony-mode
       (irony--mode-enter)
@@ -520,16 +527,17 @@ Note: WORK-DIR is not used when the compile option
 (defun irony-install-server (command)
   "Install or reinstall the Irony server.
 
-The installation requires CMake and the libclang developpement package."
+The installation requires CMake and the libclang development package."
   (interactive
    (list (let ((command
                 (format
-                 (concat "%s %s %s && %s --build . "
+                 (concat "%s %s %s %s && %s --build . "
                          "--use-stderr --config Release --target install")
                  (shell-quote-argument irony-cmake-executable)
                  (shell-quote-argument (concat "-DCMAKE_INSTALL_PREFIX="
                                                (expand-file-name
                                                 irony-server-install-prefix)))
+                 (mapconcat 'shell-quote-argument irony-extra-cmake-args " ")
                  (shell-quote-argument
                   (or irony-server-source-dir
                       (expand-file-name "server"
@@ -656,8 +664,8 @@ list (and undo information is not kept).")
 `irony--quote-strings' will convert it to \"\" instead of <SPC>.
 That is:
 
-  (irony--quote-strings \'(\"a\" \"\" \"b\"))            => \"a \\\"\\\" b\"
-  (combine-and-quote-strings \'(\"a\" \"\" \"b\"))       => \"a  b\"
+  (irony--quote-strings \\'(\"a\" \"\" \"b\"))            => \"a \\\"\\\" b\"
+  (combine-and-quote-strings \\'(\"a\" \"\" \"b\"))       => \"a  b\"
 "
   (let* ((sep (or separator " "))
          (re (concat "[\\\"]" "\\|" (regexp-quote sep))))
@@ -745,7 +753,9 @@ That is:
     (cond
      ((irony--buffer-state-modified new) 'set-unsaved)
      ((null old) nil)                   ;noop
-     ((irony--buffer-state-modified old) 'reset-unsaved))))
+     ((and
+       (irony--buffer-state-modified old)
+       (irony--buffer-state-exists old)) 'reset-unsaved))))
 
 (irony-iotask-define-task irony--t-set-unsaved
   "`set-unsaved' server command."
@@ -876,6 +886,18 @@ old-state can be nil if the old state isn't known."
            (irony--server-send-command "get-type" line col))
   :update irony--server-query-update)
 
+(defun irony--completion-line-column (&optional pos)
+  (save-excursion
+    (when pos
+      (goto-char pos))
+    ;; `position-bytes' to handle multibytes and 'multicolumns' (i.e
+    ;; tabulations) characters properly
+    (irony--without-narrowing
+      (cons
+       (line-number-at-pos)
+       (1+ (- (position-bytes (point))
+              (position-bytes (pos-bol))))))))
+
 (defun irony--get-type-task (&optional buffer pos)
   (let ((line-column (irony--completion-line-column pos)))
     (irony-iotask-chain
@@ -894,6 +916,12 @@ old-state can be nil if the old state isn't known."
         (message "%s (aka '%s')" (car types) (cadr types))
       (message "%s" (car types)))))
 
+(defun irony-parse-buffer-async (&optional callback)
+  "Parse the current buffer sending results to an optional
+  CALLBACK function."
+  (irony--run-task-asynchronously (irony--parse-task)
+                                  (or callback #'ignore)))
+
 (provide 'irony)
 
 ;; Local Variables:
